// STANDARD FOR NOSTUB COMMENT AND OTHER EXPORTED STATIC DATA EXTENSIONS
// Reference implementation version 0.92 beta
// Implemented specification version: 1.0

// This program tries to test the entire data extension API.

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

#define ENABLE_ERROR_RETURN   // Enable Returning Errors to AMS

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files
#include "dataext.h"

// Main Function
void _main(void)
{
  ESI argptr=top_estack;
  if (*argptr==END_TAG) ER_throw(ER_TOO_FEW_ARGS);
  if (*argptr!=STR_TAG) ER_throw(ER_ARG_MUST_BE_STRING);
  SYM_STR filename=GetSymstrArg(argptr);
  if (*argptr!=END_TAG) ER_throw(ER_TOO_MANY_ARGS);
  clrscr();
  SYM_ENTRY *symptr=SymFindPtr(filename,0);
  if (!symptr) ER_throw(ER_UNDEFINED_VAR);
  HANDLE h=symptr->handle;
  if (!h) ER_throw(ER_UNDEFINED_VAR);
  int locked=HeapGetLock(h);
  void *fptr=HLock(h)+2;
  unsigned char oldfont=FontSetSys(F_6x8);
  DATA_EXT *deptr=DataExtensionPointer(fptr);
  printf("File pointer at: 0x%lp\n"
         "Data extension pointer\nat: 0x%lp\n"
         "Expected at: 0x%lp\n",fptr,deptr,fptr+18);
  if (deptr) {
    printf("Spec version: %u.%u.%u.%u\n",
           (unsigned)deptr->standard_revision.major,
           (unsigned)deptr->standard_revision.minor,
           (unsigned)deptr->standard_revision.revision,
           (unsigned)deptr->standard_revision.subrev);
    printf("%u extensions",deptr->num_extensions);
    ngetchx();
    for (unsigned i=0;i<deptr->num_extensions;i++) {
      DATA_EXT_TYPE exttype=deptr->extensions[i].type_of_extension;
      void *p1=fptr+deptr->extensions[i].offset_of_extension;
      void *p2=SearchDataExtension(deptr,exttype);
      void *p3=SearchDataExtensionEx(deptr,fptr,exttype);
      clrscr();
      printf("Extension n%u\n"
             "Extension type ID: %u\n"
             "Extension type:\n%s\n"
             "Data offset: %u\n"
             "Data pointer: 0x%lp\n"
             "SrchDExt ptr: 0x%lp\n"
             "SrchDExtEx ptr: 0x%lp",i,exttype,(exttype==DE_COMMENT?
             "DE_COMMENT":(exttype==DE_PROGRAM_NAME?"DE_PROGRAM_NAME":(exttype==
             DE_VERSION_STRING?"DE_VERSION_STRING":(exttype==DE_VERSION_NUMBER?
             "DE_VERSION_NUMBER":(exttype==DE_BW_ICON?"DE_BW_ICON":(exttype==
             DE_GRAYSCALE_ICON?"DE_GRAYSCALE_ICON":(exttype==DE_INCOMPAT_FLAGS?
             "DE_INCOMPAT_FLAGS":(exttype==DE_AUTHORS?"DE_AUTHORS":
             (exttype<0x8000?"(reserved)":"(shell-specific)"
             ))))))))),deptr->extensions[i].offset_of_extension,p1,p2,p3);
      ngetchx();
    }
    clrscr();
    const char *comment1=GetDEComment(deptr);
    const char *comment2=GetDECommentEx(deptr,"(no comment)");
    printf("Comment pointer: 0x%lp\n"
           "Comment:\n%s",comment1,comment2);
    ngetchx();
    clrscr();
    const char *progname1=GetDEProgramName(deptr);
    const char *progname2=GetDEProgramNameEx(deptr,"(no program name)");
    printf("Program name ptr: 0x%lp\n"
           "Program name:\n%s",progname1,progname2);
    ngetchx();
    clrscr();
    const char *memerrptr="(memory error)";
    const char *verstr1=GetDEVersionString(deptr);
    const char *verstr2;
    TRY
      verstr2=GetDEVersionStringThrow(deptr);
    ONERR
      verstr2=memerrptr;
    ENDTRY
    const char *verstrnum1=GetDEVersionStringAndNumber(deptr);
    const char *verstrnum2;
    TRY
      verstrnum2=GetDEVersionStringAndNumberThrow(deptr);
    ONERR
      verstrnum2=memerrptr;
    ENDTRY
    printf("Version string (no throw):\n%s\n"
           "Version string (throw):\n%s\n"
           "Version str&num (no thrw):\n%s\n"
           "Version str&num (throw):\n%s\n",verstr1?:"NULL",
           verstr2?:"NULL",verstrnum1?:"NULL",verstrnum2?:"NULL");
    if (verstr1) free((char*)verstr1);
    if (verstr2&&verstr2!=memerrptr) free((char*)verstr2);
    if (verstrnum1) free((char*)verstrnum1);
    if (verstrnum2&&verstrnum2!=memerrptr) free((char*)verstrnum2);
    ngetchx();
    clrscr();
    VERSION_NUMBER *vernum=GetDEVersionNumber(deptr);
    if (!vernum) printf("No version number.\n");
    else printf("Version number: %u.%u.%u.%u\n",(unsigned)vernum->major,
                (unsigned)vernum->minor,(unsigned)vernum->revision,
                (unsigned)vernum->subrev);
    INCOMPAT_FLAGS incompat=GetDEIncompatFlags(deptr);
    printf("Incompatibility flags:\n0x%lx\n"
           "%s handles\n%s traps\n%s vectors\n%s EV_hook\n"
           "%s all stack",incompat.all_flags,incompat.creates_handles?"Creates":
           "Doesn\'t create",incompat.uses_traps?"Uses":"Doesn\'t use",
           incompat.uses_vectors?"Uses":"Doesn\'t use",incompat.uses_ev_hook?
           "Uses":"Doesn\'t use",incompat.needs_all_stack?"Needs":"Doesn\'t "
           "need");
    ngetchx();
    clrscr();
    if (!DrawDEBWIcon(deptr,LCD_MEM,10,10)) {
      PortRestore();
      DrawChar(10,10,'X',A_NORMAL);
    }
    if (!DrawDEScaledBWIcon(deptr,LCD_MEM,10,50,8,8)) {
      PortRestore();
      DrawChar(10,50,'X',A_NORMAL);
    }
    if (!DrawDEScaledBWIcon(deptr,LCD_MEM,50,10,32,32)) {
      PortRestore();
      DrawChar(50,10,'X',A_NORMAL);
    }
    ngetchx();
    if (!GrayOn()) {
      FontSetSys(oldfont);
      if (!locked) HeapUnlock(h);
      ER_throw(ER_MEMORY);
    }
    void *dplane=GrayGetPlane(DARK_PLANE);
    PortSet(dplane,239,127);
    ClrScr();
    void *lplane=GrayGetPlane(LIGHT_PLANE);
    PortSet(lplane,239,127);
    ClrScr();
    if (!DrawDEGrayscaleIcon(deptr,dplane,lplane,10,10)) {
      PortSet(dplane,239,127);
      DrawChar(10,10,'X',A_NORMAL);
      PortSet(lplane,239,127);
      DrawChar(10,10,'X',A_NORMAL);
    }
    if (!DrawDEScaledGrayscaleIcon(deptr,dplane,lplane,10,50,8,8)) {
      PortSet(dplane,239,127);
      DrawChar(10,50,'X',A_NORMAL);
      PortSet(lplane,239,127);
      DrawChar(10,50,'X',A_NORMAL);
    }
    if (!DrawDEScaledGrayscaleIcon(deptr,dplane,lplane,50,10,32,32)) {
      PortSet(dplane,239,127);
      DrawChar(50,10,'X',A_NORMAL);
      PortSet(lplane,239,127);
      DrawChar(50,10,'X',A_NORMAL);
    }
    ngetchx();
    GrayOff();
    clrscr();
    const char *authname1=GetDEAuthors(deptr);
    const char *authname2=GetDEAuthorsEx(deptr,"(unknown authors)");
    printf("Author name ptr: 0x%lp\n"
           "Author(s):\n%s",authname1,authname2);
    ngetchx();
  } else ngetchx();
  FontSetSys(oldfont);
  if (!locked) HeapUnlock(h);
}
