/*
 * This file is part of the Scale2x project.
 *
 * Copyright (C) 2001, 2002, 2003, 2004 Andrea Mazzoleni
 * Copyright (C) 2005 Kevin Kofler
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * Alternatively at the previous license terms, you are allowed to use this
 * code in your program with these conditions:
 * - the program is not used in commercial activities.
 * - the whole source code of the program is released with the binary.
 * - derivative works of the program are allowed.
 */

/* WARNING: The exception notice in License.txt does NOT apply to this file! */

#include "dataext.h"
#include <tigcclib.h>

typedef unsigned char scale2x_uint8;

static inline void scale2x_8_def_whole(scale2x_uint8* restrict dst0, scale2x_uint8* restrict dst1, const scale2x_uint8* restrict src0, const scale2x_uint8* restrict src1, const scale2x_uint8* restrict src2, unsigned count)
{
	/* first pixel */
	if (src0[0] != src2[0] && src1[0] != src1[1]) {
		dst0[0] = src1[0] == src0[0] ? src0[0] : src1[0];
		dst0[1] = src1[1] == src0[0] ? src0[0] : src1[0];
		dst1[0] = src1[0] == src2[0] ? src2[0] : src1[0];
		dst1[1] = src1[1] == src2[0] ? src2[0] : src1[0];
	} else {
		dst0[0] = src1[0];
		dst0[1] = src1[0];
		dst1[0] = src1[0];
		dst1[1] = src1[0];
	}
	++src0;
	++src1;
	++src2;
	dst0 += 2;
	dst1 += 2;

	/* central pixels */
	count -= 2;
	while (count) {
		if (src0[0] != src2[0] && src1[-1] != src1[1]) {
			dst0[0] = src1[-1] == src0[0] ? src0[0] : src1[0];
			dst0[1] = src1[1] == src0[0] ? src0[0] : src1[0];
			dst1[0] = src1[-1] == src2[0] ? src2[0] : src1[0];
			dst1[1] = src1[1] == src2[0] ? src2[0] : src1[0];
		} else {
			dst0[0] = src1[0];
			dst0[1] = src1[0];
			dst1[0] = src1[0];
			dst1[1] = src1[0];
		}

		++src0;
		++src1;
		++src2;
		dst0 += 2;
		dst1 += 2;
		--count;
	}

	/* last pixel */
	if (src0[0] != src2[0] && src1[-1] != src1[0]) {
		dst0[0] = src1[-1] == src0[0] ? src0[0] : src1[0];
		dst0[1] = src1[0] == src0[0] ? src0[0] : src1[0];
		dst1[0] = src1[-1] == src2[0] ? src2[0] : src1[0];
		dst1[1] = src1[0] == src2[0] ? src2[0] : src1[0];
	} else {
		dst0[0] = src1[0];
		dst0[1] = src1[0];
		dst1[0] = src1[0];
		dst1[1] = src1[0];
	}
}

#define SSDST(bits, num) (scale2x_uint##bits *)dst##num
#define SSSRC(bits, num) (const scale2x_uint##bits *)src##num

static void stage_scale2x(void* dst0, void* dst1, const void* src0, const void* src1, const void* src2, unsigned pixel_per_row)
{
	scale2x_8_def_whole(SSDST(8,0), SSDST(8,1), SSSRC(8,0), SSSRC(8,1), SSSRC(8,2), pixel_per_row);
}

#define SCDST(i) (dst+(i)*dst_slice)
#define SCSRC(i) (src+(i)*src_slice)

static inline void scale2x(void* void_dst, unsigned dst_slice, const void* void_src, unsigned src_slice, unsigned width, unsigned height)
{
	unsigned char* dst = (unsigned char*)void_dst;
	const unsigned char* src = (const unsigned char*)void_src;
	unsigned count;

	count = height;

	stage_scale2x(SCDST(0), SCDST(1), SCSRC(0), SCSRC(0), SCSRC(1), width);

	dst = SCDST(2);

	count -= 2;
	while (count) {
		stage_scale2x(SCDST(0), SCDST(1), SCSRC(0), SCSRC(1), SCSRC(2), width);

		dst = SCDST(2);
		src = SCSRC(1);

		--count;
	}

	stage_scale2x(SCDST(0), SCDST(1), SCSRC(0), SCSRC(1), SCSRC(1), width);
}

static inline void scale2x_ICON(const ICON *src, unsigned char dest[32][32])
{
 unsigned char srcx[16][16];
 for (unsigned i=0;i<16;i++)
  for (unsigned j=0;j<16;j++)
   srcx[i][j]=peek_bit((unsigned char*)src+(signed)(i+i+(j>>3)),(7-j)&7);
 scale2x(dest,32,srcx,16,16,16);
}

static inline void ScaleSprite16_32_OR(ICON *iconptr, void *plane,
                                       unsigned short x, unsigned short y)
{
 unsigned char buf[32][32];
 scale2x_ICON(iconptr,buf);
 for (unsigned i=0;i<32;i++)
  for (unsigned j=0;j<32;j++)
   if (buf[i][j])
    poke_bset(plane+(signed)((y+i)*30+((x+j)>>3)),7-(x+j));
}

static inline void ScaleSprite16_8_OR(ICON *iconptr, void *plane,
                                      unsigned short x, unsigned short y)
{
 for (unsigned i=0;i<8;i++)
  for (unsigned j=0;j<8;j++)
   if (peek_bit((unsigned char*)iconptr+(signed)((i<<2)+(j>>2)),(7-j-j)&7))
    poke_bset(plane+(signed)((y+i)*30+((x+j)>>3)),7-(x+j));
}

void ScaleSprite16_OR(ICON *iconptr, void *plane, unsigned short x,
                      unsigned short y, unsigned short hsize,
                      unsigned short vsize)
{
 if (hsize==32 && vsize==32) ScaleSprite16_32_OR(iconptr,plane,x,y);
 else if (hsize==8 && vsize==8) ScaleSprite16_8_OR(iconptr,plane,x,y);
 else ER_throw(ER_ARG_MISMATCH);
}
